import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:forme/forme.dart';

typedef FormeCupertinoBasePickerTriggerBuilder<
        T extends FormeCupertinoBasePickerState>
    = Widget Function(T state);

abstract class FormeCupertinoBasePicker<T,
    E extends FormeCupertinoBasePickerState<T>> extends FormeField<T> {
  FormeCupertinoBasePicker({
    required T initialValue,
    String? name,
    bool readOnly = false,
    Key? key,
    int? order,
    bool quietlyValidate = false,
    Duration? asyncValidatorDebounce,
    AutovalidateMode? autovalidateMode,
    FormeFieldStatusChanged<T>? onStatusChanged,
    FormeFieldInitialized<T>? onInitialized,
    FormeFieldSetter<T>? onSaved,
    FormeValidator<T>? validator,
    FormeAsyncValidator<T>? asyncValidator,
    FormeFieldDecorator<T>? decorator,
    bool enabled = true,
    FormeFieldValidationFilter<T>? validationFilter,
    FocusNode? focusNode,
    bool requestFocusOnUserInteraction = true,
    required this.triggerBuilder,
    this.cancelWidget,
    this.confirmWidget,
    this.anchorPoint,
    this.barrierColor,
    this.barrierDismissible,
    this.filter,
    this.height,
    this.routeSettings,
    this.semanticsDismissible,
    this.useRootNavigator,
  }) : super.allFields(
          comparator: null,
          requestFocusOnUserInteraction: requestFocusOnUserInteraction,
          focusNode: focusNode,
          validationFilter: validationFilter,
          enabled: enabled,
          decorator: decorator,
          quietlyValidate: quietlyValidate,
          asyncValidatorDebounce: asyncValidatorDebounce,
          autovalidateMode: autovalidateMode,
          onStatusChanged: onStatusChanged,
          onInitialized: onInitialized,
          onSaved: onSaved,
          validator: validator,
          asyncValidator: asyncValidator,
          order: order,
          key: key,
          name: name,
          readOnly: readOnly,
          initialValue: initialValue,
          builder: (state) {
            return triggerBuilder.call(state as E);
          },
        );

  @override
  FormeCupertinoBasePickerState<T> createState();

  final Widget? cancelWidget;
  final Widget? confirmWidget;
  final double? height;
  final Offset? anchorPoint;
  final RouteSettings? routeSettings;
  final bool? semanticsDismissible;
  final bool? useRootNavigator;
  final bool? barrierDismissible;
  final ImageFilter? filter;
  final Color? barrierColor;
  final FormeCupertinoBasePickerTriggerBuilder<E> triggerBuilder;
}

abstract class FormeCupertinoBasePickerState<T> extends FormeFieldState<T> {
  @override
  FormeCupertinoBasePicker<T, FormeCupertinoBasePickerState<T>> get widget =>
      super.widget
          as FormeCupertinoBasePicker<T, FormeCupertinoBasePickerState<T>>;

  VoidCallback? get showPicker => readOnly ? null : _showPicker;

  @protected
  Widget createPickWidget();

  @protected
  void onConfirmed();

  void _showPicker() {
    showCupertinoModalPopup(
        context: context,
        barrierColor: widget.barrierColor ?? kCupertinoModalBarrierColor,
        filter: widget.filter,
        barrierDismissible: widget.barrierDismissible ?? true,
        useRootNavigator: widget.useRootNavigator ?? true,
        semanticsDismissible: widget.semanticsDismissible,
        routeSettings: widget.routeSettings,
        anchorPoint: widget.anchorPoint,
        builder: (context) {
          return CupertinoActionSheet(
            actions: [
              Container(
                height: widget.height ?? 216,
                padding: const EdgeInsets.only(top: 6.0),
                margin: EdgeInsets.only(
                  bottom: MediaQuery.of(context).viewInsets.bottom,
                ),
                color: CupertinoColors.systemBackground.resolveFrom(context),
                child: SafeArea(
                  top: false,
                  child: createPickWidget(),
                ),
              ),
              CupertinoActionSheetAction(
                isDestructiveAction: true,
                child: widget.confirmWidget ?? const Text('Done'),
                onPressed: () {
                  Navigator.of(context).pop();
                  onConfirmed();
                },
              ),
            ],
            cancelButton: CupertinoActionSheetAction(
              isDefaultAction: true,
              child: widget.cancelWidget ?? const Text('Cancel'),
              onPressed: () {
                Navigator.of(context).pop();
              },
            ),
          );
        });
  }
}
