import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:forme/forme.dart';
import '../forme_datetime_type.dart';
import 'base_picker.dart';

class FormeCupertinoDateField
    extends FormeCupertinoBasePicker<DateTime?, FormeCupertinoDateFieldState> {
  FormeCupertinoDateField({
    bool enabled = true,
    DateTime? initialValue,
    String? name,
    bool readOnly = false,
    Key? key,
    int? order,
    bool quietlyValidate = false,
    Duration? asyncValidatorDebounce,
    AutovalidateMode? autovalidateMode,
    FormeFieldStatusChanged<DateTime?>? onStatusChanged,
    FormeFieldInitialized<DateTime?>? onInitialized,
    FormeFieldSetter<DateTime?>? onSaved,
    FormeValidator<DateTime?>? validator,
    FormeAsyncValidator<DateTime?>? asyncValidator,
    FormeFieldDecorator<DateTime?>? decorator,
    FormeFieldValidationFilter<DateTime?>? validationFilter,
    FocusNode? focusNode,
    required FormeCupertinoBasePickerTriggerBuilder<
            FormeCupertinoDateFieldState>
        triggerBuilder,
    Offset? anchorPoint,
    Color? barrierColor,
    bool? barrierDismissible,
    ImageFilter? filter,
    bool requestFocusOnUserInteraction = true,
    double height = 216,
    RouteSettings? routeSettings,
    bool? semanticsDismissible,
    bool? useRootNavigator,
    Widget? cancelWidget,
    Widget? confirmWidget,
    this.dateOrder,
    this.maximumDate,
    this.maximumYear,
    this.minimumDate,
    this.minimumYear,
    this.type = FormeDateTimeType.date,
    this.use24hFormat,
    this.minuteInterval = 1,
    this.backgroundColor,
  }) : super(
          cancelWidget: cancelWidget,
          confirmWidget: confirmWidget,
          focusNode: focusNode,
          validationFilter: validationFilter,
          enabled: enabled,
          readOnly: readOnly,
          decorator: decorator,
          quietlyValidate: quietlyValidate,
          asyncValidatorDebounce: asyncValidatorDebounce,
          autovalidateMode: autovalidateMode,
          onStatusChanged: onStatusChanged,
          onInitialized: onInitialized,
          onSaved: onSaved,
          validator: validator,
          asyncValidator: asyncValidator,
          order: order,
          key: key,
          name: name,
          initialValue: initialValue,
          triggerBuilder: triggerBuilder,
          barrierColor: barrierColor,
          barrierDismissible: barrierDismissible,
          filter: filter,
          height: height,
          routeSettings: routeSettings,
          requestFocusOnUserInteraction: requestFocusOnUserInteraction,
          semanticsDismissible: semanticsDismissible,
          useRootNavigator: useRootNavigator,
          anchorPoint: anchorPoint,
        );

  @override
  FormeCupertinoDateFieldState createState() => FormeCupertinoDateFieldState();
  final int minuteInterval;
  final FormeDateTimeType type;
  final Color? backgroundColor;
  final int? maximumYear;
  final int? minimumYear;
  final DateTime? maximumDate;
  final DateTime? minimumDate;
  final bool? use24hFormat;
  final DatePickerDateOrder? dateOrder;
}

class FormeCupertinoDateFieldState
    extends FormeCupertinoBasePickerState<DateTime?> {
  DateTime? _time;

  @override
  FormeCupertinoDateField get widget => super.widget as FormeCupertinoDateField;

  CupertinoDatePickerMode get _mode => widget.type == FormeDateTimeType.date
      ? CupertinoDatePickerMode.date
      : CupertinoDatePickerMode.dateAndTime;

  @override
  void onConfirmed() {
    if (_time != null) {
      didChange(_time);
      requestFocusOnUserInteraction();
    }
  }

  @override
  Widget createPickWidget() {
    _time = null;
    return CupertinoDatePicker(
        mode: _mode,
        maximumDate: widget.maximumDate,
        maximumYear: widget.maximumYear,
        minimumDate: widget.minimumDate,
        minimumYear: widget.minimumYear ?? 1,
        minuteInterval: widget.minuteInterval,
        use24hFormat: widget.use24hFormat ?? false,
        dateOrder: widget.dateOrder,
        backgroundColor: widget.backgroundColor,
        initialDateTime: value,
        onDateTimeChanged: (time) {
          _time = time;
        });
  }

  @override
  DateTime? get value {
    final DateTime? value = super.value;
    if (value == null) {
      return null;
    }
    return _simple(value);
  }

  DateTime _simple(DateTime dateTime) {
    switch (widget.type) {
      case FormeDateTimeType.date:
        return DateTime(dateTime.year, dateTime.month, dateTime.day);
      case FormeDateTimeType.dateTime:
        return DateTime(dateTime.year, dateTime.month, dateTime.day,
            dateTime.hour, dateTime.minute);
    }
  }
}
