import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:forme/forme.dart';
import 'base_picker.dart';

class FormeCupertinoPicker
    extends FormeCupertinoBasePicker<int, FormeCupertinoPickerState> {
  FormeCupertinoPicker({
    int? initialValue,
    String? name,
    bool readOnly = false,
    Key? key,
    int? order,
    bool quietlyValidate = false,
    Duration? asyncValidatorDebounce,
    AutovalidateMode? autovalidateMode,
    FormeFieldStatusChanged<int>? onStatusChanged,
    FormeFieldInitialized<int>? onInitialized,
    FormeFieldSetter<int>? onSaved,
    FormeValidator<int>? validator,
    FormeAsyncValidator<int>? asyncValidator,
    FormeFieldDecorator<int>? decorator,
    bool enabled = true,
    FormeFieldValidationFilter<int>? validationFilter,
    FocusNode? focusNode,
    bool requestFocusOnUserInteraction = true,
    Widget? cancelWidget,
    Widget? confirmWidget,
    Color? barrierColor,
    bool? barrierDismissible,
    ImageFilter? filter,
    double height = 216,
    RouteSettings? routeSettings,
    bool? semanticsDismissible,
    bool? useRootNavigator,
    Offset? anchorPoint,
    required FormeCupertinoBasePickerTriggerBuilder<FormeCupertinoPickerState>
        triggerBuilder,
    this.backgroundColor,
    this.diameterRatio,
    required this.itemExtent,
    this.looping = true,
    this.magnification,
    this.offAxisFraction,
    this.selectionOverlay,
    this.squeeze,
    this.useMagnifier,
    required this.children,
  }) : super(
          requestFocusOnUserInteraction: requestFocusOnUserInteraction,
          focusNode: focusNode,
          validationFilter: validationFilter,
          enabled: enabled,
          decorator: decorator,
          quietlyValidate: quietlyValidate,
          asyncValidatorDebounce: asyncValidatorDebounce,
          autovalidateMode: autovalidateMode,
          onStatusChanged: onStatusChanged,
          onInitialized: onInitialized,
          onSaved: onSaved,
          validator: validator,
          asyncValidator: asyncValidator,
          order: order,
          key: key,
          name: name,
          readOnly: readOnly,
          initialValue: initialValue ?? 0,
          cancelWidget: cancelWidget,
          confirmWidget: cancelWidget,
          triggerBuilder: triggerBuilder,
          barrierColor: barrierColor,
          barrierDismissible: barrierDismissible,
          filter: filter,
          height: height,
          routeSettings: routeSettings,
          semanticsDismissible: semanticsDismissible,
          useRootNavigator: useRootNavigator,
          anchorPoint: anchorPoint,
        );

  @override
  FormeCupertinoPickerState createState() => FormeCupertinoPickerState();

  final double? diameterRatio;
  final Color? backgroundColor;
  final double? offAxisFraction;
  final bool? useMagnifier;
  final double? magnification;
  final double? squeeze;
  final bool looping;
  final Widget? selectionOverlay;
  final double itemExtent;
  final List<Widget> children;
}

class FormeCupertinoPickerState extends FormeCupertinoBasePickerState<int> {
  late FixedExtentScrollController _scrollController;

  @override
  FormeCupertinoPicker get widget => super.widget as FormeCupertinoPicker;

  late int _index;

  @override
  void onConfirmed() {
    didChange(_index);
    requestFocusOnUserInteraction();
  }

  @override
  Widget createPickWidget() {
    WidgetsBinding.instance.addPostFrameCallback((timestamp) {
      _scrollController.jumpToItem(value);
    });
    return CupertinoPicker(
      scrollController: _scrollController,
      diameterRatio: widget.diameterRatio ?? 1.07,
      backgroundColor: widget.backgroundColor,
      offAxisFraction: widget.offAxisFraction ?? 0.0,
      useMagnifier: widget.useMagnifier ?? false,
      magnification: widget.magnification ?? 1.0,
      squeeze: widget.squeeze ?? 1.45,
      looping: widget.looping,
      selectionOverlay: widget.selectionOverlay ??
          const CupertinoPickerDefaultSelectionOverlay(),
      itemExtent: widget.itemExtent,
      onSelectedItemChanged: (index) => _index = index,
      children: widget.children,
    );
  }

  @override
  void initStatus() {
    super.initStatus();
    _index = initialValue;
    _scrollController = FixedExtentScrollController(initialItem: _index);
  }

  @override
  void onStatusChanged(FormeFieldChangedStatus<int> status) {
    super.onStatusChanged(status);
    if (status.isValueChanged) {
      _index = status.value;
      _scrollController.jumpToItem(_index);
    }
  }

  @override
  void dispose() {
    _scrollController.dispose();
    super.dispose();
  }
}
