import 'package:flutter/cupertino.dart';
import 'package:forme/forme.dart';

class FormeCupertinoSegmentedControl<T extends Object> extends FormeField<T?> {
  FormeCupertinoSegmentedControl({
    String? name,
    T? initialValue,
    bool readOnly = false,
    Key? key,
    int? order,
    bool quietlyValidate = false,
    Duration? asyncValidatorDebounce,
    AutovalidateMode? autovalidateMode,
    FormeFieldStatusChanged<T?>? onStatusChanged,
    FormeFieldInitialized<T?>? onInitialized,
    FormeFieldSetter<T?>? onSaved,
    FormeValidator<T?>? validator,
    FormeAsyncValidator<T?>? asyncValidator,
    FormeFieldDecorator<T?>? decorator,
    bool requestFocusOnUserInteraction = true,
    bool enabled = true,
    FormeFieldValidationFilter<T?>? validationFilter,
    FocusNode? focusNode,
    FormeValueComparator<T?>? comparator,
    required this.children,
    this.borderColor,
    this.disableBorderColor,
    this.disableSelectedColor,
    this.disableUnselectedColor,
    this.padding,
    this.pressedColor,
    this.selectedColor,
    this.unselectedColor,
  }) : super.allFields(
            comparator: comparator,
            focusNode: focusNode,
            validationFilter: validationFilter,
            enabled: enabled,
            requestFocusOnUserInteraction: requestFocusOnUserInteraction,
            decorator: decorator,
            quietlyValidate: quietlyValidate,
            asyncValidatorDebounce: asyncValidatorDebounce,
            autovalidateMode: autovalidateMode,
            onStatusChanged: onStatusChanged,
            onInitialized: onInitialized,
            onSaved: onSaved,
            validator: validator,
            asyncValidator: asyncValidator,
            order: order,
            name: name,
            initialValue: initialValue,
            readOnly: readOnly,
            key: key,
            builder: (state) {
              final bool readOnly = state.readOnly;
              return Focus(
                focusNode: state.focusNode,
                //still now way to disable CupertinoSegmentedControl officially after 3 years....
                child: IgnorePointer(
                  ignoring: readOnly,
                  child: CupertinoSegmentedControl<T>(
                      groupValue: state.value,
                      children: children,
                      unselectedColor:
                          readOnly ? disableUnselectedColor : unselectedColor,
                      selectedColor:
                          readOnly ? disableSelectedColor : selectedColor,
                      borderColor: readOnly ? disableBorderColor : borderColor,
                      pressedColor: pressedColor,
                      padding: padding,
                      onValueChanged: (v) {
                        state.didChange(v);
                        state.requestFocusOnUserInteraction();
                      }),
                ),
              );
            });

  final Map<T, Widget> children;
  final Color? unselectedColor;
  final Color? selectedColor;
  final Color? borderColor;
  final Color? pressedColor;
  final EdgeInsetsGeometry? padding;
  final Color? disableUnselectedColor;
  final Color? disableSelectedColor;
  final Color? disableBorderColor;
}
