import 'package:flutter/cupertino.dart';
import 'package:forme/forme.dart';

class FormeCupertinoSlider extends FormeField<double> {
  FormeCupertinoSlider({
    String? name,
    double? initialValue,
    bool readOnly = false,
    Key? key,
    int? order,
    bool quietlyValidate = false,
    Duration? asyncValidatorDebounce,
    AutovalidateMode? autovalidateMode,
    FormeFieldStatusChanged<double>? onStatusChanged,
    FormeFieldInitialized<double>? onInitialized,
    FormeFieldSetter<double>? onSaved,
    FormeValidator<double>? validator,
    FormeAsyncValidator<double>? asyncValidator,
    FormeFieldDecorator<double>? decorator,
    bool enabled = true,
    FormeFieldValidationFilter<double>? validationFilter,
    FocusNode? focusNode,
    bool requestFocusOnUserInteraction = true,
    required this.min,
    required this.max,
    this.activeColor,
    this.divisions,
    this.onChangeEnd,
    this.onChangeStart,
    this.onChanged,
    this.thumbColor = CupertinoColors.white,
  }) : super.allFields(
          requestFocusOnUserInteraction: requestFocusOnUserInteraction,
          focusNode: focusNode,
          validationFilter: validationFilter,
          enabled: enabled,
          order: order,
          decorator: decorator,
          key: key,
          readOnly: readOnly,
          name: name,
          initialValue: initialValue ?? min,
          quietlyValidate: quietlyValidate,
          asyncValidatorDebounce: asyncValidatorDebounce,
          autovalidateMode: autovalidateMode,
          onStatusChanged: onStatusChanged,
          onInitialized: onInitialized,
          onSaved: onSaved,
          validator: validator,
          asyncValidator: asyncValidator,
          comparator: null,
          builder: (baseState) {
            final _FormeCupertinoSliderState state =
                baseState as _FormeCupertinoSliderState;
            return ValueListenableBuilder<double?>(
                valueListenable: state.notifier,
                builder: (context, _, __) {
                  return Focus(
                    focusNode: state.focusNode,
                    child: CupertinoSlider(
                        value: state.value,
                        min: min,
                        max: max,
                        onChangeStart: (v) {
                          state.focusNode.requestFocus();
                          onChangeStart?.call(v);
                        },
                        onChangeEnd: (v) {
                          state.didChange(v);
                          onChangeEnd?.call(v);
                        },
                        activeColor: activeColor,
                        thumbColor: thumbColor,
                        divisions: divisions ?? (max - min).floor(),
                        onChanged: state.readOnly
                            ? null
                            : (v) {
                                state.updateValue(v);
                                onChanged?.call(v);
                              }),
                  );
                });
          },
        );

  final double min;
  final double max;
  final ValueChanged<double>? onChangeStart;
  final ValueChanged<double>? onChangeEnd;
  final ValueChanged<double>? onChanged;
  final Color? activeColor;
  final Color thumbColor;
  final int? divisions;

  @override
  FormeFieldState<double> createState() => _FormeCupertinoSliderState();
}

class _FormeCupertinoSliderState extends FormeFieldState<double> {
  @override
  FormeCupertinoSlider get widget => super.widget as FormeCupertinoSlider;

  late final ValueNotifier<double?> notifier;

  void updateValue(double value) {
    notifier.value = value;
  }

  @override
  void initStatus() {
    super.initStatus();
    notifier = FormeMountedValueNotifier(null);
  }

  @override
  double get initialValue {
    final double defaultInitialValue = widget.initialValue;
    if (defaultInitialValue < widget.min) {
      return widget.min;
    }
    if (defaultInitialValue > widget.max) {
      return widget.max;
    }
    return defaultInitialValue;
  }

  @override
  double get value => notifier.value ?? super.value;

  @override
  void dispose() {
    notifier.dispose();
    super.dispose();
  }

  @override
  void onStatusChanged(FormeFieldChangedStatus<double> status) {
    super.onStatusChanged(status);
    if (status.isValueChanged) {
      notifier.value = null;
    }
  }
}
