import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:forme/forme.dart';
import 'base_picker.dart';

class FormeCupertinoTimerField
    extends FormeCupertinoBasePicker<Duration?, FormeCupertinoTimerFieldState> {
  FormeCupertinoTimerField({
    bool enabled = true,
    Duration? initialValue,
    String? name,
    bool readOnly = false,
    Key? key,
    int? order,
    bool quietlyValidate = false,
    Duration? asyncValidatorDebounce,
    AutovalidateMode? autovalidateMode,
    FormeFieldStatusChanged<Duration?>? onStatusChanged,
    FormeFieldInitialized<Duration?>? onInitialized,
    FormeFieldSetter<Duration?>? onSaved,
    FormeValidator<Duration?>? validator,
    FormeAsyncValidator<Duration?>? asyncValidator,
    FormeFieldDecorator<Duration?>? decorator,
    FormeFieldValidationFilter<Duration?>? validationFilter,
    FocusNode? focusNode,
    bool requestFocusOnUserInteraction = true,
    required FormeCupertinoBasePickerTriggerBuilder<
            FormeCupertinoTimerFieldState>
        triggerBuilder,
    Offset? anchorPoint,
    Color? barrierColor,
    bool? barrierDismissible,
    ImageFilter? filter,
    double height = 216,
    RouteSettings? routeSettings,
    bool? semanticsDismissible,
    bool? useRootNavigator,
    this.minuteInterval = 1,
    this.secondInterval = 1,
    this.mode = CupertinoTimerPickerMode.hms,
    this.alignment,
    this.backgroundColor,
  }) : super(
          focusNode: focusNode,
          validationFilter: validationFilter,
          enabled: enabled,
          readOnly: readOnly,
          decorator: decorator,
          quietlyValidate: quietlyValidate,
          asyncValidatorDebounce: asyncValidatorDebounce,
          autovalidateMode: autovalidateMode,
          onStatusChanged: onStatusChanged,
          onInitialized: onInitialized,
          onSaved: onSaved,
          validator: validator,
          asyncValidator: asyncValidator,
          order: order,
          key: key,
          name: name,
          initialValue: initialValue,
          triggerBuilder: triggerBuilder,
          barrierColor: barrierColor,
          barrierDismissible: barrierDismissible,
          filter: filter,
          height: height,
          routeSettings: routeSettings,
          requestFocusOnUserInteraction: requestFocusOnUserInteraction,
          semanticsDismissible: semanticsDismissible,
          useRootNavigator: useRootNavigator,
          anchorPoint: anchorPoint,
        );

  @override
  FormeCupertinoTimerFieldState createState() =>
      FormeCupertinoTimerFieldState();

  final int minuteInterval;
  final int secondInterval;
  final CupertinoTimerPickerMode mode;
  final Color? backgroundColor;
  final AlignmentGeometry? alignment;
}

class FormeCupertinoTimerFieldState
    extends FormeCupertinoBasePickerState<Duration?> {
  Duration? _duration;

  @override
  FormeCupertinoTimerField get widget =>
      super.widget as FormeCupertinoTimerField;

  @override
  void onConfirmed() {
    if (_duration == null) {
      if (value == null) {
        didChange(Duration.zero);
        requestFocusOnUserInteraction();
      }
    } else {
      didChange(_duration);
      requestFocusOnUserInteraction();
    }
    _duration = null;
  }

  @override
  Widget createPickWidget() {
    _duration = null;
    return CupertinoTimerPicker(
      minuteInterval: widget.minuteInterval,
      secondInterval: widget.secondInterval,
      backgroundColor: widget.backgroundColor,
      alignment: widget.alignment ?? Alignment.center,
      initialTimerDuration: value ?? Duration.zero,
      mode: widget.mode,
      onTimerDurationChanged: (Duration timer) {
        _duration = timer;
      },
    );
  }
}
