import 'package:flutter/material.dart';

import 'package:forme/forme.dart';

class FormeCheckbox extends FormeField<bool?> {
  FormeCheckbox({
    bool? initialValue,
    String? name,
    bool readOnly = false,
    Key? key,
    int? order,
    bool quietlyValidate = false,
    Duration? asyncValidatorDebounce,
    AutovalidateMode? autovalidateMode,
    FormeFieldStatusChanged<bool?>? onStatusChanged,
    FormeFieldInitialized<bool?>? onInitialized,
    FormeFieldSetter<bool?>? onSaved,
    FormeValidator<bool?>? validator,
    FormeAsyncValidator<bool?>? asyncValidator,
    bool requestFocusOnUserInteraction = true,
    FormeFieldDecorator<bool?>? decorator,
    bool enabled = true,
    FormeFieldValidationFilter<bool?>? validationFilter,
    FocusNode? focusNode,
    this.tristate = false,
    this.activeColor,
    this.autofocus = false,
    this.checkColor,
    this.fillColor,
    this.focusColor,
    this.hoverColor,
    this.materialTapTargetSize,
    this.mouseCursor,
    this.overlayColor,
    this.shape,
    this.side,
    this.splashRadius,
    this.visualDensity,
  }) : super.allFields(
          comparator: null,
          focusNode: focusNode,
          validationFilter: validationFilter,
          enabled: enabled,
          decorator: decorator,
          requestFocusOnUserInteraction: requestFocusOnUserInteraction,
          order: order,
          quietlyValidate: quietlyValidate,
          asyncValidatorDebounce: asyncValidatorDebounce,
          autovalidateMode: autovalidateMode,
          onStatusChanged: onStatusChanged,
          onInitialized: onInitialized,
          onSaved: onSaved,
          validator: validator,
          asyncValidator: asyncValidator,
          key: key,
          readOnly: readOnly,
          name: name,
          initialValue: tristate ? initialValue : initialValue ?? false,
          builder: (state) {
            final bool readOnly = state.readOnly;
            final bool? value = state.value;
            return Checkbox(
              autofocus: autofocus,
              focusNode: state.focusNode,
              side: side,
              tristate: tristate,
              mouseCursor: mouseCursor,
              shape: shape,
              activeColor: activeColor,
              fillColor: fillColor,
              checkColor: checkColor,
              materialTapTargetSize: materialTapTargetSize,
              focusColor: focusColor,
              hoverColor: hoverColor,
              overlayColor: overlayColor,
              splashRadius: splashRadius,
              visualDensity: visualDensity,
              value: value,
              isError: state.errorText != null,
              onChanged: readOnly
                  ? null
                  : (value) {
                      state.didChange(value);
                      state.requestFocusOnUserInteraction();
                    },
            );
          },
        );

  final bool tristate;
  final Color? activeColor;
  final MouseCursor? mouseCursor;
  final MaterialStateProperty<Color?>? fillColor;
  final Color? checkColor;
  final Color? focusColor;
  final Color? hoverColor;
  final MaterialStateProperty<Color?>? overlayColor;
  final double? splashRadius;
  final VisualDensity? visualDensity;
  final MaterialTapTargetSize? materialTapTargetSize;
  final OutlinedBorder? shape;
  final bool autofocus;
  final BorderSide? side;

  @override
  FormeFieldState<bool?> createState() => _FormeCheckboxState();
}

class _FormeCheckboxState extends FormeFieldState<bool?> {
  @override
  FormeCheckbox get widget => super.widget as FormeCheckbox;

  @override
  void didChange(bool? newValue) {
    if (newValue == null && !widget.tristate) {
      throw Exception(
          'current value can not be null, set tristate to true if you want to support nullable');
    }
    super.didChange(newValue);
  }
}
